'Build Version: 3.0.104.0
'//////////////////////////////////////////////////////////////////////
'//      THIS CODE AND INFORMATION IS PROVIDED TO YOU FOR YOUR REFERENTIAL 
'//      PURPOSES ONLY, AND IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY REM  KIND, 
'//      EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
'//      WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE, 
'//      AND MAY NOT BE REDISTRIBUTED IN ANY MANNER.
'//  
'//      Copyright (c) Microsoft Corporation. All rights reserved.
'//
'// Script File:
'//
'//   SetConnIP.vbs
'//
'//   WMI script to set the IP settings for a single connection.
'//
'// Notes:
'//
'//   Usage:
'//
'//   cscript //nologo SetConnIP.vbs <ConnName> [ DHCP | STATIC <IPSettings>] [DBG]
'//
'//     IPSettings = [IPAddr=<AddrList>] [Mask=<MaskList>]
'//                  [DefGW=<GWList>] [GWMetric=<GWMetricList>]
'//
'// Inputs:
'//
'//   ConnName - Connection name to perform operation on
'//   Action - DHCP or STATIC
'//   AddrList - comma-separated list of IP addresses
'//   MaskList - command-separated list of IP address masks
'//   GWList - comma-separated list of default gateway IP addresses
'//   GWMetriclist - comma-separated list of integer default gateway metrics
'//
'//   - ConnName and Action are required.
'//   - IPAddr and Mask must appear together.
'//   - DefGW and GWMetric must appear together.
'//   - AddrList and MaskList must have the same number of elements.
'//   - GWList and GWMetricList must have the same number of elements.
'//   - For all lists, do not use any whitespace between elements.
'//   - For connection names with whitespace, put double-quotes around
'//     the name.  E.g., "Local Area Connection"
'//   - DBG enables debug messages.
'//
'// Outputs:
'//
'//   ERRORLEVEL - 0 = success, reboot not required.
'//                1 = success, reboot required.
'//                > 1 = failure
'//
'// Environment variables referenced:
'//
'//   None.
'//
'// Runs in:
'//
'//   W2K, WXP, N32, N64
'//
'//////////////////////////////////////////////////////////////////////

' LogFile Constants
Const For_APPENDING = 8
Const INFO = 1
Const FAIL = 2
Const WARN = 3
on error resume next
TimerStart = Timer
set WshShell = CreateObject("WScript.Shell")
' Create a Log File at %windir%\debug\
Set wshSysEnv = WshShell.Environment("PROCESS")
Set oFs = CreateObject("Scripting.FileSystemObject")
Set LogF = oFs.OpenTexTFile(WshSysEnv("SYSTEMROOT") & "\debug\setconip.Log",For_APPENDING,True)
If IsObject(LogF)= False Then
	WScript.Echo "Unable to create LogFile."
	WScript.Quit(2)
End if
WriteLog "Startig setconIP", INFO


RetSts = main ()
'WScript.Sleep(2000)
TimeElapsed = Cdbl(Mid(CStr(Timer - TimerStart),1,5))
WriteLog "Ending setconIP. Processing took: " & TimeElapsed & " seconds.", INFO
WScript.Quit(RetSts)


'//////////////////////////////////////////////////////////////////////
'//
'// main
'//
'//////////////////////////////////////////////////////////////////////

Function main ()

  on error resume next

  ' Make sure we have enough arguments.
  ' Must have at least 2: ConnName and Action to perform.

  if WScript.Arguments.count < 2 Then
    WriteLog("Insufficient arguments." & vbCrLf),FAIL
    call Usage()
    main = 2
    exit function
  End If

  ' Parse the command line parameters.

  RebootReq = 0
  ConnName = Null
  UConnName = Null
  Action = Null
  AddrList = Null
  MaskList = Null
  GWList = Null
  GWMetricList = Null
  DebugMode = False

  For i = 0 To WScript.Arguments.count-1
    ArgName = split (WScript.Arguments(i), "=")
    UArgName = UCase (ArgName(0))

    ' The connection name must be the first argument.

    if i = 0 then
      ConnName = ArgName(0)
      UConnName = UArgName

    ' The action must be the second argument.

    elseif i = 1 then
      Action = UArgName

    ' Everything else is position-independent.

    else
      select case UArgName

        case "IPADDR"
          if UBound(ArgName) < 1 Then
            WriteLog ("Missing IP address list for IPAddr parameter." & vbCrLf),FAIL
            call Usage ()
            main = 2
            exit function
          End If
          AddrList = split (ArgName(1), ",")

        case "MASK"
          if UBound(ArgName) < 1 Then
            WriteLog ("Missing mask list for Mask parameter." & vbCrLf),Fail
            call Usage ()
            main = 2
            exit function
          End If
          MaskList = split (ArgName(1), ",")

        case "DEFGW"
          if UBound(ArgName) < 1 Then
            WriteLog ("Missing default gateway list for DefGW parameter." & vbCrLf),FAIL
            call Usage ()
            main = 2
            exit function
          End If
          GWList = split (ArgName(1), ",")

        case "GWMETRIC"
          if UBound(ArgName) < 1 Then
            WriteLog ("Missing default gateway metric list for GWMetric parameter." & vbCrLf),FAIL
            call Usage ()
            main = 2
            exit function
          End If
          GWMetricList = split (ArgName(1), ",")

        case "DBG"
          DebugMode=True
		
		Case "DNS"
			if UBound(ArgName) < 1 Then
            WriteLog ("Missing DNS Server list for DNS parameter." & vbCrLf),FAIL
            call Usage ()
            main = 2
            exit function
          End If
          DNSServerList = split (ArgName(1), ",")

        case else
          WriteLog ("Unknown command line argument." & vbCrLf), FAIL
          call Usage ()
          main = 2
          exit function

      end select

    end if
  Next

  ' Validate Action.

  if Action = "STATIC" then

    ' If Action is STATIC, validate the IPSettings parameters.
    ' IPAddr and Mask must exist together.
    ' DefGW and GWMetric must exist together.
    WriteLog "Validating Input for Setting Static IP Configuration.",INFO

    if (not IsNull (AddrList) AND IsNull (MaskList)) OR _
       (not IsNull (MaskList) AND IsNull (AddrList)) then
      WriteLog ("Missing Mask with IPAddr or missing IPAddr with Mask." & vbCrLf),FAIL
      call Usage ()
      main = 2
      exit function
    end if

    if (not IsNull (GWList) AND IsNull (GWMetricList)) OR _
       (not IsNull (GWMetricList) AND IsNull (GWList)) then
      WriteLog ("Missing GWMetric with DefGW or missing DefGW with GWMetric." & vbCrLf),FAIL
      call Usage ()
      main = 2
      exit function
    end If
    
    if IsNull (DNSServerList) then
        WriteLog ("Need to specify at least one DNS Server."),FAIL
        main = 2
        exit function
    end if

    ' Make sure the number of elements in each list match.

    if not IsNull (AddrList) AND not IsNull (MaskList) then
      if UBound (AddrList) <> UBound (MaskList) then
        WriteLog ("Number of elements in IPAddr AddrList doesn't match Mask MaskList."),FAIL
        main = 2
        exit function
      end if
    end if

    if not IsNull (GWList) AND not IsNull (GWMetricList) then
      if UBound (GWList) <> UBound (GWMetricList) then
        WriteLog ("Number of elements in DefGW GWList doesn't match GWMetric GWMetricList."),FAIL
        main = 2
        exit function
      end if
    end If

    


  elseif Action <> "DHCP" then

    ' Not STATIC and not DHCP, must be something unexpected.

    WriteLog ("Invalid Action.  Must be either STATIC or DHCP." & vbCrLf),FAIL
    call Usage ()
    main = 2
    exit function

  end If

If Action = "DHCP" Then  WriteLog "DHCP Configuration selected.",INFO

  ' Debug display info.

  if DebugMode = True then

    WScript.echo ("Action    : '" & Action & "'")
    WScript.echo ("ConnName  : '" & ConnName & "'")
    WScript.echo ("UConnName : '" & UConnName & "'")

    MsgStr = ""
    For i = LBound(AddrList) to UBound(AddrList)
      if i = 0 then
        MsgStr = AddrList(i)
      else
        MsgStr = MsgStr & ", " & AddrList(i)
      end if
    Next
    WriteLog ("AddrList  : '" & MsgStr & "'"),INFO

    MsgStr = ""
    For i = LBound(MaskList) to UBound(MaskList)
      if i = 0 then
        MsgStr = MaskList(i)
      else
        MsgStr = MsgStr & ", " & MaskList(i)
      end if
    Next
    WriteLog ("MaskList  : '" & MsgStr & "'"),INFO

    MsgStr = ""
    For i = LBound(GWList) to UBound(GWList)
      if i = 0 then
        MsgStr = GWList(i)
      else
        MsgStr = MsgStr & ", " & GWList(i)
      end if
    Next
    WriteLog ("GWList    : '" & MsgStr & "'"),INFO	

    MsgStr = ""
    For i = LBound(GWMetricList) to UBound(GWMetricList)
      if i = 0 then
        MsgStr = GWMetricList(i)
      else
        MsgStr = MsgStr & ", " & GWMetricList(i)
      end if
    Next
    WriteLog ("GWMetricList  : '" & MsgStr & "'"),INFO
  

	MsgStr = ""
    For i = LBound(DNSServerList) to UBound(DNSServerList)
      if i = 0 then
        MsgStr = DNSServerList(i)
      else
        MsgStr = MsgStr & ", " & DNSServerList(i)
      end if
    Next
    WriteLog ("DNSServer(s)  : '" & MsgStr & "'"),INFO
  end if


  ' Get the collection of NetworkAdapters.

  err.clear
  WriteLog "Setting IP Configuration.",INFO
  Set NetworkAdapterSet = GetObject("winmgmts:{impersonationLevel=impersonate}").ExecQuery("SELECT PNPDeviceID, Index FROM Win32_NetworkAdapter WHERE Manufacturer != 'Microsoft'")
    
  if IsNull (NetworkAdapterSet) OR IsEmpty (NetworkAdapterSet) then
    WriteLog ("Failed to get collection of network adapters."),FAIL
    WriteLog ("Error info - err.number: " & err.number & "; '" & err.description + "'"),FAIL
    main = 2
    exit Function
  Else
	  WriteLog "Successfully bound to Win32_NetworkAdapter Object.",INFO
  end if

  ' Undefine the connection NIC index.
  ' Loop through the list of adapters looking for the connection name as
  ' defined in the registry.

  ConnIndex = Null
  For Each NetworkAdapter In NetworkAdapterSet
    If (Not IsNull(NetworkAdapter) and (Len(NetworkAdapter.PnPDeviceID) > 0)) Then

      ' The PnpID leads to the Driver and ClassGUID info.
      ' The Driver leads to the Network instance ID.
      ' The Network instand ID and the ClassGUID leads to the connection name.

      DriverID = WshShell.RegRead("HKLM\System\CurrentControlSet\Enum\" & NetworkAdapter.PNPDeviceID & "\Driver")
      ClassGUID = WshShell.RegRead("HKLM\System\CurrentControlSet\Enum\" & NetworkAdapter.PNPDeviceID & "\ClassGUID")
      NetCfgInstanceID = WshShell.RegRead("HKLM\System\CurrentControlSet\Control\Class\" & DriverID & "\NetCfgInstanceId")
      NetConnectionName = WshShell.RegRead("HKLM\System\CurrentControlSet\Control\Network\" & ClassGUID & "\" & NetCfgInstanceId & "\Connection\Name")

      UNetConnName = UCase(NetConnectionName)

      ' If we found the connection, save the index and exit the loop.

      if (UConnName = UNetConnName) Then
        WriteLog "Found Adapter specified in Commandline (" & UConnName & ").",INFO
        ConnIndex = NetworkAdapter.Index
        Exit For
      End If

    End If
  Next

  ' Free the NetworkAdapter collection object.

  Set NetworkAdapterSet = Nothing

  ' If we didn't find the connection, log the error and bail.

  if IsNull (ConnIndex) Then
    WriteLog ("Failed to find network connection named '" & ConnName & "'"),FAIL
    main = 2
    exit function
  End If

  ' Get the NetworkAdapterConfiguration object instance for this connection index.
  WriteLog "Binding to Win32_NetworkAdapterConfiguration Object.",INFO
  Set NetCfg = GetObject("winmgmts:{impersonationLevel=impersonate}").Get("Win32_NetworkAdapterConfiguration.Index=" & CStr(ConnIndex))


  ' Update the connection based on Action.

  '//////////////////////////////////////////////////
  '
  ' Set the connection to DHCP.

  if Action = "DHCP" then

    ' Because EnableDHCP can timeout with a return status of 81,
    ' loop at most 6 times delaying 10 seconds between calls.

    LoopIdx = 1
    WriteLog "Setting Adapter to DHCP.",INFO

    Do
	
      ' Perform the update.
	  err.clear
      RetSts = NetCfg.EnableDHCP ()

      if err.number <> 0 then
        WriteLog ("Failed EnableDHCP call."),FAIL
        WriteLog ("Error info - err.number: " & err.number & "; '" & err.description + "'"),FAIL
        main = 2
        exit function
      end if

      ' Check return status.
      ' If an 81, then increment the retry loop counter and sleep for 10 seconds.

      if RetSts = 81 then
        WriteLog ("Timeout " & CStr(LoopIdx) & ". Sleeping 10 seconds before retrying."),WARN
        LoopIdx = LoopIdx + 1
        WScript.Sleep(10000)

      ' Some other error.

      elseif RetSts > 1 then
        WriteLog ("Failed to set connection to DHCP." & vbCrLf & _
                      "Win32_NetworkAdapterConfiguration.EnableDHCP return status = " & CStr(RetSts)),FAIL
        main = RetSts
        exit function

      ' Return status is either success or success with reboot required.

      else

        ' If reboot required status returned,
        ' make sure we signal the caller the same way.

        if RetSts = 1 then RebootReq = 1
        exit do

      end if

    Loop While LoopIdx <= 6

    ' If we exhausted the do loop and still didn't set the values, bail.

    if LoopIdx >= 6 AND RetSts > 1 then
      WriteLog ("Failed to set connection to DHCP after 1 minute." & vbCrLf & _
                    "Win32_NetworkAdapterConfiguration.EnableDHCP return status = " & CStr(RetSts)),FAIL
      main = RetSts
      exit function
    end if


  '//////////////////////////////////////////////////
  '
  ' Set the connection to STATIC IP.

  elseif Action = "STATIC" then

    ' Set the IP address(es) for the connection.
	WriteLog "Setting Adapter to STATIC IP Configuration.",INFO

    if not IsNull (AddrList) AND not IsNull (MaskList) then

      ' Because EnableStatic can timeout with a return status of 81,
      ' loop at most 6 times delaying 10 seconds between calls.

      LoopIdx = 1
      Do

        ' Perform the update.
		WritLog "Now updating the IP Address and the Subnetmask.",INFO
        err.clear
        RetSts = NetCfg.EnableStatic (AddrList, MaskList)      
		
        if err.number <> 0 then
          WriteLog ("Failed EnableStatic call."),FAIL
          WScript.echo ("Error info - err.number: " & err.number & "; '" & err.description + "'"),FAIL
          main = 2
          exit function
        end if

        ' Check return status.
        ' If an 81, then increment the retry loop counter and sleep for 10 seconds.

        if RetSts = 81 then
          WriteLog ("Timeout " & CStr(LoopIdx) & ". Sleeping 10 seconds before retrying."),WARN
          LoopIdx = LoopIdx + 1
          WScript.Sleep(10000)

        ' Some other error.

        elseif RetSts > 1 then
          WriteLog ("Failed to set connection static IP settings." & vbCrLf & _
                        "Win32_NetworkAdapterConfiguration.EnableStatic return status = " & CStr(RetSts)),FAIL
          main = RetSts
          exit function

	        ' Return status is either success or success with reboot required.
	      
        else

          ' If reboot required status returned,
          ' make sure we signal the caller the same way.

          if RetSts = 1 then RebootReq = 1
          exit do

        end if

      Loop While LoopIdx <= 6

      ' If we exhausted the do loop and still didn't set the values, bail.

      if LoopIdx >= 6 AND RetSts > 1 then
        WriteLog ("Failed to set connection static IP settings after 1 minute." & vbCrLf & _
                      "Win32_NetworkAdapterConfiguration.EnableStatic return status = " & CStr(RetSts)),FAIL
        main = RetSts
        exit function
      end If
	 
    end if

    ' Set the default gateway(s) for the connection.

    if not IsNull (GWList) and not IsNull (GWMetricList) Then
	  WriteLog "Setting Default Gateway.",INFO
      err.clear
      RetSts = NetCfg.SetGateways (GWList, GWMetricList)

      if err.number <> 0 then
        WriteLog ("Failed SetGateways call."),FAIL
        WriteLog ("Error info - err.number: " & err.number & "; '" & err.description + "'"),FAIL
        main = 2
        exit function
      end if

      if RetSts > 1 then
        WriteLog ("Failed to set connection default gateway settings." & vbCrLf & _
                      "Win32_NetworkAdapterConfiguration.SetGateways return status = " & CStr(RetSts)),FAIL
        main = RetSts
        exit function
      end if

      ' If reboot required status returned,
      ' make sure we signal the caller the same way.

      if RetSts = 1 then RebootReq = 1
    end If
	
	' Now enable DNS
	WriteLog "Setting DNS Servers.",INFO
	RetSts =  netCfg.SetDNSServerSearchOrder(DNSServerList)
	WriteLog "Setting DNS Returned: " & RetSts,INFO
	
  else
    WriteLog ("Unexpected error. Action value invalid."),FAIL
    main = 2
    exit function
  end if


  ' Display debug information.

  if DebugMode = True Then
	WriteLog "Printing Debug Information.",INFO
    Set NetCfg = Nothing
    Set NetCfg = GetObject("winmgmts:{impersonationLevel=impersonate}").Get("Win32_NetworkAdapterConfiguration.Index=" & CStr(ConnIndex))


    ' Display the connection's description and MAC address.

    WriteLog("Description                  = " & NetCfg.Description)
    WriteLog("MACAddress                   = " & NetCfg.MACAddress)

    ' Display the connection's IP settings.

    WriteLog(vbCrLf &  "IP Settings:" & vbCrLf)

    WriteLog("DHCPEnabled                  = " & CStr(NetCfg.DHCPEnabled))
    WriteLog("DHCPLeaseExpires             = " & CStr(NetCfg.DHCPLeaseExpires))
    WriteLog("DHCPLeaseObtained            = " & CStr(NetCfg.DHCPLeaseObtained))
    WriteLog("DHCPServer                   = " & NetCfg.DHCPServer)

    WriteLog("IPEnabled                    = " & CStr(NetCfg.IPEnabled))
    WriteLog("IPAddress:")
    For i = LBound(NetCfg.IPAddress) To UBound(NetCfg.IPAddress)
      WriteLog("                             = " & NetCfg.IPAddress(i))
    Next
    WriteLog("IPSubnet:")
    For i = LBound(NetCfg.IPSubnet) To UBound(NetCfg.IPSubnet)
      WriteLog("                             = " & NetCfg.IPSubnet(i))
    Next
    WriteLog("DefaultIPGateway:")
    For i = LBound(NetCfg.DefaultIPGateway) To UBound(NetCfg.DefaultIPGateway)
      WriteLog("                             = " & NetCfg.DefaultIPGateway(i))
    Next
    WriteLog("GatewayCostMetric:")
    For i = LBound(NetCfg.GatewayCostMetric) To UBound(NetCfg.GatewayCostMetric)
      WriteLog("                             = " & NetCfg.GatewayCostMetric(i))
    Next
    WriteLog("IPConnectionMetric           = " & CStr(NetCfg.IPConnectionMetric))

    ' Display the connection's DNS settings.

    WriteLog(vbCrLf &  "DNS Settings:" & vbCrLf)

    WriteLog("DNSDomain                    = " & NetCfg.DNSDomain)
    WriteLog("DNSHostName                  = " & NetCfg.DNSHostName)
    WriteLog("DomainDNSRegistrationEnabled = " & CStr(NetCfg.DomainDNSRegistrationEnabled))
    WriteLog("FullDNSRegistrationEnabled   = " & CStr(NetCfg.FullDNSRegistrationEnabled))
    WriteLog("DNSServerSearchOrder:")
    For i = LBound(NetCfg.DNSServerSearchOrder) To UBound(NetCfg.DNSServerSearchOrder)
      WriteLog("                             = " & NetCfg.DNSServerSearchOrder(i))
    Next
    WriteLog("DNSDomainSuffixSearchOrder:")
    For i = LBound(NetCfg.DNSDomainSuffixSearchOrder) To UBound(NetCfg.DNSDomainSuffixSearchOrder)
      WriteLog("                             = " & NetCfg.DNSDomainSuffixSearchOrder(i))
    Next

    ' Display the connections WINS settings.

    WriteLog(vbCrLf &  "WINS Settings:" & vbCrLf)

    WriteLog("TcpipNetbiosOptions          = " & CStr(NetCfg.TcpipNetbiosOptions))
    WriteLog("DNSEnabledForWINSResolution  = " & CStr(NetCfg.DNSEnabledForWINSResolution))
    WriteLog("WINSEnableLMHostsLookup      = " & CStr(NetCfg.WINSEnableLMHostsLookup))
    WriteLog("WINSHostLookupFile           = " & NetCfg.WINSHostLookupFile)
    WriteLog("WINSPrimaryServer            = " & NetCfg.WINSPrimaryServer)
    WriteLog("WINSSecondaryServer          = " & NetCfg.WINSSecondaryServer)

  end if

  ' Return with appropriate status.
  ' Check for reboot required.

  main = 0
  if RebootReq = 1 then main = RebootReq

End Function


'//////////////////////////////////////////////////////////////////////
'//
'// Usage
'//
'//////////////////////////////////////////////////////////////////////

Sub Usage ()
  Dim strUsage
  
  
  strUsage = "Usage:" & vbCrLf
  strUsage =  strUsage & "  cscript //nologo SetConnIP.vbs <ConnName> [ DHCP | STATIC <IPSettings>]" & vbCrLf
  strUsage =  strUsage & "    IPSettings = [IPAddr=<AddrList>] [Mask=<MaskList>]"
  strUsage =  strUsage & "                 [DefGW=<GWList>] [GWMetric=<GWMetricList>]" & vbCrLf
  strUsage =  strUsage & "Examples:" & vbCrLf
  strUsage =  strUsage & "  SetConnIP.vbs ""Local Area Connection"" DHCP"  & vbCrLf
  strUsage =  strUsage & "  SetConnIP.vbs FrontNet STATIC IPAddr=10.1.1.1,10.1.1.2 Mask=255.255.255.0,255.255.255.0"  & vbCrLf
  strUsage =  strUsage & "  SetConnIP.vbs FrontNet STATIC IPAddr=192.168.2.1 Mask=255.255.255.224 DefGW=192.168.2.28 GWMetric=1"
  
  WScript.StdOut.WriteLine strUsage
  WriteLog strUsage 
End Sub


Function WriteLog(strLogString,intSev)
	Select Case intSev
		Case INFO
			strLogString = "INFORMATION: " & Now() & ": " & strLogString
		Case WARN
			strLogString = "WARNING: " & Now() & ": " & strLogString
		Case FAIL
			strLogString = "ERROR: " & Now() & ": " & strLogString						
		Case Else
			strLogString = "-----: " & Now() & ": " & strLogString						
	End Select
	If IsObject(logF) Then
		logF.WriteLine strLogString
	End If
	WScript.Echo strLogString
End Function
